#ifndef _common_h_
#define _common_h_

#include <sstream>
#include <string.h>

#ifndef SAFE_DELETE
#define SAFE_DELETE(p) { if( p!=NULL ) {delete p; p=NULL; } }
#endif

#ifndef SAFE_FREE
#define SAFE_FREE(p) { if( p != NULL ) { free(p); p=NULL ; } }
#endif

// Simple ToString function meant to be used by standard object types
template<typename T>
static std::string ToString(T value) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

template<typename T>
static T ToValue(const std::string & value) {
	std::stringstream ss(value);
	T ret;
	ss >> ret;
	return ret;
}

class CUtilities {
public:
	static unsigned char SetBit(unsigned char data, unsigned char bit, bool value);
	static bool GetBit(unsigned char data, unsigned char bit);

	static bool IsNumber(std::string s);

	static bool ValidateIPAddress(std::string ipAddress);
};

class CConvert {
public:
	static std::string ToCamelCase(std::string value);
	static std::string UndoCamelCase(std::string value, char spaceOrSymbol, bool allLowerCase);

	static bool IPAddressToByteString(std::string ipAddress, uint8_t * byteString, const uint8_t maxByteStringLength);
};

class CFileIo {
public:
	static bool MkDir(std::string path);
	static bool ExistFile(std::string path);
	static bool AppendFile(std::string path, const char * buffer, size_t bufferLength);
	static bool RemoveFile(std::string path);

	static time_t GetFileTime(std::string path);

	static int GetNumberOfFiles(std::string path);
	static std::string GetOldestFileInDirectory(std::string path, const char * dir, const char * pattern);

	static std::string GetWorkingDirectory();
};

#endif // _common_h_