#include "CASBACnetStackExampleDatabase.h"

#ifdef WIN32
#include <winsock2.h>
#include <iphlpapi.h>
#pragma comment(lib, "IPHLPAPI.lib")

#define MALLOC(x) HeapAlloc(GetProcessHeap(), 0, (x))
#define FREE(x) HeapFree(GetProcessHeap(), 0, (x))
#endif // WIN32

ExampleDatabase::ExampleDatabase() {
	this->Reset();
}

ExampleDatabase::~ExampleDatabase() {
	this->Reset();
}

void ExampleDatabase::Reset() {
	this->DeviceName = "Device_Rainbow";
	this->AnalogInputName = "AnalogInput_Red";
	this->AnalogOutputName = "AnalogOutput_Vermilion";
	this->AnalogValueName = "AnalogValue_Orange";
	this->BinaryInputName = "BinaryInput_Amber";
	this->BinaryOutputName = "BinaryOutput_Yellow";
	this->BinaryValueName = "BinaryValue_Chartreuse";
	this->MultiStateInputName = "MultiStateInput_Green";
	this->MultiStateOutputName = "MultiStateOutput_Teal";
	this->MultiStateValueName = "MultiStateValue_Blue";
	this->BitstringValueName = "BitstringValue_White";
	this->CharacterstringValueName = "CharacterstringValue_Gray";
	this->DateValueName = "DateValue_Brown";
	this->IntegerValueName = "IntegerValue_Violet";
	this->LargeAnalogValueName = "LargeAnalogValue_Purple";
	this->OctetstringValueName = "OctetstringValue_Black";
	this->PositiveIntegerValueName = "PositiveIntegerValue_Magenta";
	this->TimeValueName = "TimeValue_Pink";
	this->NetworkPortName = "NetworkPort_Charcoal";

	this->AnalogInputPresentValue = 1.23f;
	memset(this->AnalogOutputPriorityArrayValues, 0, sizeof(float) * 16);
	memset(this->AnalogOutputPriorityArrayNulls, true, sizeof(bool) * 16);
	this->AnalogValuePresentValue = 5.43f;
	this->BinaryInputPresentValue = 0;
	memset(this->BinaryOutputPriorityArrayValues, 0, sizeof(uint8_t) * 16);
	memset(this->BinaryOutputPriorityArrayNulls, true, sizeof(bool) * 16);
	this->BinaryValuePresentValue = 0;
	this->MultiStateInputPresentValue = 1;
	memset(this->MultiStateOutputPriorityArrayValues, 0, sizeof(uint32_t) * 16);
	memset(this->MultiStateOutputPriorityArrayNulls, true, sizeof(bool) * 16);
	this->MultiStateValuePresentValue = 1;
	memset(this->BitstringValuePresentValue, 0, sizeof(bool) * 256);
	this->BitstringValuePresentValue[0] = true;
	this->BitstringValuePresentValue[1] = false;
	this->BitstringValuePresentValue[2] = false;
	this->BitstringValuePresentValue[3] = true;
	this->BitstringValuePresentValueLength = 4;
	this->CharacterStringValuePresentValue = "hello world";
	this->DateValuePresentValueYear = 118;
	this->DateValuePresentValueMonth = 3;
	this->DateValuePresentValueDay = 15;
	this->DateValuePresentValueWeekday = 4;
	this->IntegerValuePresentValue = -12345;
	this->LargeAnalogValuePresentValue = 123456789.85;
	memset(this->OctetStringValuePresentValue, 0, sizeof(uint8_t) * 256);
	this->OctetStringValuePresentValueLength = 0;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x00;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x01;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x02;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x03;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x04;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x05;
	this->OctetStringValuePresentValue[OctetStringValuePresentValueLength++] = 0x06;
	this->PositiveIntegerValuePresentValue = 54321;
	this->TimeValuePresentValueHour = 15;
	this->TimeValuePresentValueMinute = 13;
	this->TimeValuePresentValueSecond = 55;
	this->TimeValuePresentValueHundrethSecond = 0;

	this->DeviceUTCOffset = 0;
	this->DeviceCurrentTimeOffset = 0;

	this->NetworkPortBACnetIPUDPPort = 47808;

	memset(this->NetworkPortIPAddress, 0, 4);
	this->NetworkPortIPAddressLength = 0;
	memset(this->NetworkPortIPDefaultGateway, 0, 4);
	this->NetworkPortIPDefaultGatewayLength;
	memset(this->NetworkPortIPSubnetMask, 0, 4);
	this->NetworkPortIPSubnetMaskLength = 0;

	this->NetworkPortIPDNSServerLength = 4;
	for (size_t i = 0; i < this->NetworkPortIPDNSServers.size(); i++) {
		delete(this->NetworkPortIPDNSServers[i]);
	}
	this->NetworkPortIPDNSServers.clear();

	memset(this->BroadcastIPAddress, 0, 4);
}

void ExampleDatabase::LoadNetworkPortProperties() {

	// This function loads the Network port property values needed.
	// It uses system functions to get values like the IP Address and stores them
	// in the example database

#ifdef WIN32
	PIP_ADAPTER_ADDRESSES pAddresses = NULL;
	PIP_ADAPTER_INFO pAdapterInfo;
	PIP_ADAPTER_INFO pAdapter = NULL;
	ULONG flags = GAA_FLAG_INCLUDE_PREFIX | GAA_FLAG_INCLUDE_GATEWAYS;

	ULONG ulOutBufLen = sizeof(IP_ADAPTER_INFO);
	ULONG outBufLen = 15000;
	DWORD dwRetVal = 0;
	pAdapterInfo = (IP_ADAPTER_INFO *)MALLOC(sizeof(IP_ADAPTER_INFO));
	if (pAdapterInfo == NULL) {
		printf("Error allocating memory needed to call GetAdaptersinfo\n");
		return;
	}

	// Make an initial call to GetAdaptersInfo to get
	// the necessary size into the ulOutBufLen variable
	if (GetAdaptersInfo(pAdapterInfo, &ulOutBufLen) == ERROR_BUFFER_OVERFLOW) {
		FREE(pAdapterInfo);
		pAdapterInfo = (IP_ADAPTER_INFO *)MALLOC(ulOutBufLen);
		if (pAdapterInfo == NULL) {
			printf("Error allocating memory needed to call GetAdaptersinfo\n");
			return;
		}
	}

	std::string selectedAdapterName;

	if ((dwRetVal = GetAdaptersInfo(pAdapterInfo, &ulOutBufLen)) == NO_ERROR) {
		pAdapter = pAdapterInfo;
		while (pAdapter) {
			// If this is the Ethernet port, then extract the parameters
			if (pAdapter->Type == MIB_IF_TYPE_ETHERNET) {
				// Ethernet adapter
				// Extract the ethernet parameters needed for the Network Port Object
				// IP Address
				this->NetworkPortIPAddressLength = sscanf_s(pAdapter->IpAddressList.IpAddress.String, "%hhd.%hhd.%hhd.%hhd", &this->NetworkPortIPAddress[0], &this->NetworkPortIPAddress[1], &this->NetworkPortIPAddress[2], &this->NetworkPortIPAddress[3]);

				// Default Gateway
				this->NetworkPortIPDefaultGatewayLength = sscanf_s(pAdapter->GatewayList.IpAddress.String, "%hhd.%hhd.%hhd.%hhd", &this->NetworkPortIPDefaultGateway[0], &this->NetworkPortIPDefaultGateway[1], &this->NetworkPortIPDefaultGateway[2], &this->NetworkPortIPDefaultGateway[3]);

				// Subnet Mask
				this->NetworkPortIPSubnetMaskLength = sscanf_s(pAdapter->IpAddressList.IpMask.String, "%hhd.%hhd.%hhd.%hhd", &this->NetworkPortIPSubnetMask[0], &this->NetworkPortIPSubnetMask[1], &this->NetworkPortIPSubnetMask[2], &this->NetworkPortIPSubnetMask[3]);

				// Interface Name
				selectedAdapterName = std::string(pAdapter->AdapterName);

				// Prepare the broadcast address
				for (size_t i = 0; i < 4; i++) {
					this->BroadcastIPAddress[i] = ((this->NetworkPortIPAddress[i] & this->NetworkPortIPSubnetMask[i]) == 0 ? 0xFF : this->NetworkPortIPAddress[i]);
				}

				break;
			}
			pAdapter = pAdapter->Next;
		}
	}

	if (pAdapterInfo) {
		FREE(pAdapterInfo);
	}

	// Get the address info, this is to get the DNS info
	pAddresses = (IP_ADAPTER_ADDRESSES *)MALLOC(outBufLen);
	if (pAddresses == NULL) {
		printf("Memory allocation failed for IP_ADAPTER_ADDRESSES struct\n");
		return;
	}

	if (GetAdaptersAddresses(AF_INET, flags, NULL, pAddresses, &outBufLen) == ERROR_BUFFER_OVERFLOW) {
		FREE(pAddresses);
		pAddresses = NULL;
		return;
	}

	PIP_ADAPTER_ADDRESSES pCurrAddresses = pAddresses;
	IP_ADAPTER_DNS_SERVER_ADDRESS *pDnServer = NULL;
	while (pCurrAddresses) {
		std::string adapterName = std::string(pCurrAddresses->AdapterName);
		if (adapterName.compare(selectedAdapterName) == 0) {

			// Extract the DNS server details
			pDnServer = pCurrAddresses->FirstDnsServerAddress;
			while (pDnServer) {
				SOCKADDR * sockaddr = pDnServer->Address.lpSockaddr;
				if (sockaddr != NULL && sockaddr->sa_family == AF_INET) {
					SOCKADDR_IN * temp = (SOCKADDR_IN*)sockaddr;
					uint8_t * dns = new uint8_t[4];
			
					sscanf_s(inet_ntoa(temp->sin_addr), "%hhd.%hhd.%hhd.%hhd", &dns[0], &dns[1], &dns[2], &dns[3]);
					this->NetworkPortIPDNSServers.push_back(dns);
				}
				pDnServer = pDnServer->Next;
			}

			break;
		}

		pCurrAddresses = pCurrAddresses->Next;
	}

	if (pAddresses) {
		FREE(pAddresses);
	}

#endif // WIN32
}
