#ifndef __CASBACnetStackExampleDatabase_h__
#define __CASBACnetStackExampleDatabase_h__

/*
The CASBACnetStackExampleDatabase is a data store that contains the
some example data used in the BACnetStackDLLExample.

This data is represented by BACnet objects for this server example.
There will be one object of each type currently supported by the CASBACnetStack.

The database will include the following:
	- present value
	- name
	- for outputs priority array (bool and value)
*/

#include <string>
#include <vector>

class ExampleDatabase {

	public:

	// Names
	std::string AnalogInputName;
	std::string AnalogOutputName;
	std::string AnalogValueName;
	std::string BinaryInputName;
	std::string BinaryOutputName;
	std::string BinaryValueName;
	//std::string CalendarName;
	//std::string CommandName;
	std::string DeviceName;
	//std::string EventEnrollmentName;
	//std::string FileName;
	//std::string GroupName;
	//std::string LoopName;
	std::string MultiStateInputName;
	std::string MultiStateOutputName;
	//std::string NotificationClassName;
	//std::string ProgramName;
	//std::string ScheduleName;
	//std::string AveragingName;
	std::string MultiStateValueName;
	//std::string TrendLogName;
	//std::string LifeSafetyPointName;
	//std::string LifeSafetyZoneName;
	//std::string AccumulatorName;
	//std::string PulseConverterName;
	//std::string EventLogName;
	//std::string GlobalGroupName;
	//std::string TrendLogMultipleName;
	//std::string LoadControlName;
	//std::string StructuredViewName;
	//std::string AccessDoorName;
	//std::string TimerName;
	//std::string AccessCredentialName;
	//std::string AccessPointName;
	//std::string AccessRightsName;
	//std::string AccessUserName;
	//std::string AccessZoneName;
	//std::string CredentialDataInputName;
	//std::string NetworkSecurityName;
	std::string BitstringValueName;
	std::string CharacterstringValueName;
	//std::string DatepatternValueName;
	std::string DateValueName;
	//std::string DatetimepatternValueName;
	//std::string DatetimeValueName;
	std::string IntegerValueName;
	std::string LargeAnalogValueName;
	std::string OctetstringValueName;
	std::string PositiveIntegerValueName;
	//std::string TimepatternValueName;
	std::string TimeValueName;
	//std::string NotificationForwarderName;
	//std::string AlertEnrollmentName;
	//std::string ChannelName;
	//std::string LightingOutputName;
	//std::string BinaryLightingOutputName;
	std::string NetworkPortName;
	//std::string ElevatorGroupName;
	//std::string EscalatorName;
	//std::string LiftName;

	// Device Properties
	int DeviceUTCOffset;
	int64_t DeviceCurrentTimeOffset;
	
	// Present Values
	float AnalogInputPresentValue;
	float AnalogOutputPriorityArrayValues[16];
	bool AnalogOutputPriorityArrayNulls[16];
	float AnalogValuePresentValue;
	uint8_t BinaryInputPresentValue;
	uint8_t BinaryOutputPriorityArrayValues[16];
	bool BinaryOutputPriorityArrayNulls[16];
	uint8_t BinaryValuePresentValue;
	uint32_t MultiStateInputPresentValue;
	uint32_t MultiStateOutputPriorityArrayValues[16];
	bool MultiStateOutputPriorityArrayNulls[16];
	uint32_t MultiStateValuePresentValue;
	bool BitstringValuePresentValue[256];
	uint16_t BitstringValuePresentValueLength;
	std::string CharacterStringValuePresentValue;
	uint8_t DateValuePresentValueYear;
	uint8_t DateValuePresentValueMonth;
	uint8_t DateValuePresentValueDay;
	uint8_t DateValuePresentValueWeekday;
	int32_t IntegerValuePresentValue;
	double LargeAnalogValuePresentValue;
	uint8_t OctetStringValuePresentValue[256];
	uint16_t OctetStringValuePresentValueLength;
	uint32_t PositiveIntegerValuePresentValue;
	uint8_t TimeValuePresentValueHour;
	uint8_t TimeValuePresentValueMinute;
	uint8_t TimeValuePresentValueSecond;
	uint8_t TimeValuePresentValueHundrethSecond;

	// Network Port Properties
	uint16_t NetworkPortBACnetIPUDPPort;
	uint8_t NetworkPortIPAddress[4];
	uint8_t NetworkPortIPAddressLength;
	uint8_t NetworkPortIPDefaultGateway[4];
	uint8_t NetworkPortIPDefaultGatewayLength;
	uint8_t NetworkPortIPSubnetMask[4];
	uint8_t NetworkPortIPSubnetMaskLength;
	std::vector<uint8_t*> NetworkPortIPDNSServers;
	uint8_t NetworkPortIPDNSServerLength;

	uint8_t BroadcastIPAddress[4];
	
	// Object Instances
	static const uint32_t INSTANCE_DEVICE = 389999;
	static const uint32_t INSTANCE_ANALOG_INPUT = 0;
	static const uint32_t INSTANCE_ANALOG_OUTPUT = 1;
	static const uint32_t INSTANCE_ANALOG_VALUE = 2;
	static const uint32_t INSTANCE_BINARY_INPUT = 3;
	static const uint32_t INSTANCE_BINARY_OUTPUT = 4;
	static const uint32_t INSTANCE_BINARY_VALUE = 5;
	//static const uint32_t INSTANCE_CALENDAR = 6;
	//static const uint32_t INSTANCE_COMMAND = 7;
	//static const uint32_t INSTANCE_EVENT_ENROLLMENT = 9;
	//static const uint32_t INSTANCE_FILE = 10;
	//static const uint32_t INSTANCE_GROUP = 11;
	//static const uint32_t INSTANCE_LOOP = 12;
	static const uint32_t INSTANCE_MULTI_STATE_INPUT = 13;
	static const uint32_t INSTANCE_MULTI_STATE_OUTPUT = 14;
	//static const uint32_t INSTANCE_NOTIFICATION_CLASS = 15;
	//static const uint32_t INSTANCE_PROGRAM = 16;
	//static const uint32_t INSTANCE_SCHEDULE = 17;
	//static const uint32_t INSTANCE_AVERAGING = 18;
	static const uint32_t INSTANCE_MULTI_STATE_VALUE = 19;
	//static const uint32_t INSTANCE_TREND_LOG = 20;
	//static const uint32_t INSTANCE_LIFE_SAFETY_POINT = 21;
	//static const uint32_t INSTANCE_LIFE_SAFETY_ZONE = 22;
	//static const uint32_t INSTANCE_ACCUMULATOR = 23;
	//static const uint32_t INSTANCE_PULSE_CONVERTER = 24;
	//static const uint32_t INSTANCE_EVENT_LOG = 25;
	//static const uint32_t INSTANCE_GLOBAL_GROUP = 26;
	//static const uint32_t INSTANCE_TREND_LOG_MULTIPLE = 27;
	//static const uint32_t INSTANCE_LOAD_CONTROL = 28;
	//static const uint32_t INSTANCE_STRUCTURED_VIEW = 29;
	//static const uint32_t INSTANCE_ACCESS_DOOR = 30;
	//static const uint32_t INSTANCE_TIMER = 31;
	//static const uint32_t INSTANCE_ACCESS_CREDENTIAL = 32;
	//static const uint32_t INSTANCE_ACCESS_POINT = 33;
	//static const uint32_t INSTANCE_ACCESS_RIGHTS = 34;
	//static const uint32_t INSTANCE_ACCESS_USER = 35;
	//static const uint32_t INSTANCE_ACCESS_ZONE = 36;
	//static const uint32_t INSTANCE_CREDENTIAL_DATA_INPUT = 37;
	//static const uint32_t INSTANCE_NETWORK_SECURITY = 38;
	static const uint32_t INSTANCE_BITSTRING_VALUE = 39;
	static const uint32_t INSTANCE_CHARACTERSTRING_VALUE = 40;
	//static const uint32_t INSTANCE_DATEPATTERN_VALUE = 41;
	static const uint32_t INSTANCE_DATE_VALUE = 42;
	//static const uint32_t INSTANCE_DATETIMEPATTERN_VALUE = 43;
	//static const uint32_t INSTANCE_DATETIME_VALUE = 44;
	static const uint32_t INSTANCE_INTEGER_VALUE = 45;
	static const uint32_t INSTANCE_LARGE_ANALOG_VALUE = 46;
	static const uint32_t INSTANCE_OCTETSTRING_VALUE = 47;
	static const uint32_t INSTANCE_POSITIVE_INTEGER_VALUE = 48;
	//static const uint32_t INSTANCE_TIMEPATTERN_VALUE = 49;
	static const uint32_t INSTANCE_TIME_VALUE = 50;
	//static const uint32_t INSTANCE_NOTIFICATION_FORWARDER = 51;
	//static const uint32_t INSTANCE_ALERT_ENROLLMENT = 52;
	//static const uint32_t INSTANCE_CHANNEL = 53;
	//static const uint32_t INSTANCE_LIGHTING_OUTPUT = 54;
	//static const uint32_t INSTANCE_BINARY_LIGHTING_OUTPUT = 55;
	static const uint32_t INSTANCE_NETWORK_PORT = 56;
	//static const uint32_t INSTANCE_ELEVATOR_GROUP = 57;
	//static const uint32_t INSTANCE_ESCALATOR = 58;
	//static const uint32_t INSTANCE_LIFT = 59;

	// Constructor / Deconstructor
	ExampleDatabase();
	~ExampleDatabase();

	void Reset();

	// Helper Functions
	
	void LoadNetworkPortProperties();

};

#endif // __CASBACnetStackExampleDatabase_h__
