#ifndef __LOGGING_H__
#define __LOGGING_H__

/*******************************************************************************
Version Table
Version		Date			Name	Notes
0.01		27 Sep 2017		ACF		Created version table.
0.02		01 Dec 2017		ACF		Fixed some compiler warnings
********************************************************************************/

/*******************************************************************************
CLogging

Simple logging functions to help logging to stdout and file
********************************************************************************/

#include <assert.h>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define CLOGGING_MAX_LINE_LENGTH 1024	// Max length of logging text

class CLogging {
private:
	static bool m_globalEnableLogging;			// Flag to enable or disable logging, set to true on startup
	static unsigned int m_globalDebugLevel;		// The debug logging level, set to Above Average (20) on startup
	static bool m_globalEnableLogToFile;		// Flag to enable or disable log to file, set to true on startup
	static std::string m_logFile;					// Pointer to the current log file, set to empty
	static struct tm m_currentLogDate;

	// Prints to the consol window. Only really happens with Windows devices. Enabled by default on windows. 
	static void LogToStdOut(const char * buffer, const unsigned short length);

	/**
	* This will append the buffer to the CLOGGING_DEFAULT_FILENAME. If the logToErrorLog is true then
	* the message will also be logged to the CLOGGING_ERROR_FILENAME as well.
	*/
	static void LogToFile(const char * buffer, const unsigned short length);

	// This is the main function that actually does the logging.
	static void LogMessage(const unsigned int logLevel, std::string prefix, const char * lpszText, va_list argList);
	static void LogSend(const unsigned int logLevel, const char * lpszText, va_list argList);

public:

	static void EnableLogging() { CLogging::m_globalEnableLogging = true; }
	static void DisableLogging() { CLogging::m_globalEnableLogging = false; }
	static void EnableLogToFile() { CLogging::m_globalEnableLogToFile = true; }
	static void DisableLogToFile() { CLogging::m_globalEnableLogToFile = false; }
	static void SetDebugLoggingLevel(unsigned int logLevel) { CLogging::m_globalDebugLevel = logLevel; }
	static void SetLoggingFile(std::string logFilename) { CLogging::m_logFile = logFilename; }
	static void SetCurrentLogDate();

	static void LogFileManagement();

	static void Error(const char * lpszText, ...);
	static void Fyi(const char * lpszText, ...);
	static void Debug(const unsigned int logLevel, const char * lpszText, ...);
	static void LogBuffer(const unsigned int logLevel, std::string buffer);

	enum debug_level {
		DEBUG_LEVEL_PANIC = 0, // Very few messags 
		DEBUG_LEVEL_CRITICAL = 1, // Error()  
		DEBUG_LEVEL_IMPORTANT = 10, // FYI() Default setting on shipping. 
		DEBUG_LEVEL_ABOVE_AVERAGE = 20,
		DEBUG_LEVEL_NORMAL = 50,
		DEBUG_LEVEL_LOW = 75,
		DEBUG_LEVEL_ALL = 100, // Lots of messages, SQL Queries 
	};
};


#endif // __LOGGING_H__